#include "ogreapplication.h"
#include <OgreErrorDialog.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interface of MyApplication
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class MyApplication : public OgreApplication,Ogre::MouseListener
{
public:
  MyApplication();

  virtual void mousePressed(Ogre::MouseEvent*) {};
  virtual void mouseReleased(Ogre::MouseEvent*);

  virtual void mouseClicked(Ogre::MouseEvent*) {};
  virtual void mouseEntered(Ogre::MouseEvent*) {};
  virtual void mouseExited(Ogre::MouseEvent*) {};
protected:
  virtual void init();
private:
  Ogre::RaySceneQuery* mRaySceneQuery;
  Ogre::Entity*        mSelected;
};

using namespace Ogre;
using namespace std;

//************************************************************
// Implementation of MyApplication
//************************************************************
MyApplication::MyApplication()
  : OgreApplication("My Application"),
    mSelected(NULL)
{
}
void
MyApplication::init()
{
  mSceneManager->setAmbientLight(ColourValue(1,1,1));
      
  Entity *ent1 = mSceneManager->createEntity("Robot1","robot.mesh");
  SceneNode* node1 = mSceneManager->getRootSceneNode()->createChildSceneNode("Robot1Node");
  node1->attachObject(ent1);

  Entity *ent2 = mSceneManager->createEntity("Ninja1","ninja.mesh");
  SceneNode* node2 = mSceneManager->getRootSceneNode()->createChildSceneNode("Ninja1Node");
  node2->attachObject(ent2);
  node2->setPosition(Vector3(50,0,0));
  node2->setScale(Vector3(0.42f,0.42f,0.42f));
  
  mRaySceneQuery = mSceneManager->createRayQuery(Ray());
  mEventProcessor->addMouseListener(this);
}
void
MyApplication::mouseReleased(Ogre::MouseEvent* e)
{
  if (e->getButtonID() == InputEvent::BUTTON0_MASK && e->isShiftDown()) // Shift+Left 
  {
    // Deselect old one if any
    if (mSelected) mSelected->getParentSceneNode()->showBoundingBox(false);
    mWindow->setDebugText("");
    // Use a ray scene query to find where we clicked
    Ray mouseRay = mCamera->getCameraToViewportRay(e->getX(),e->getY());
    mRaySceneQuery->setRay(mouseRay);
    mRaySceneQuery->setSortByDistance(true);
    RaySceneQueryResult &result = mRaySceneQuery->execute();
    if (!result.empty())
    {
      RaySceneQueryResultEntry& re = result.front();
      if (re.movable && re.movable->getMovableType() == "Entity")
      {
        mSelected = (Entity*)(re.movable);
        mSelected->getParentSceneNode()->showBoundingBox(true);
        mWindow->setDebugText("Selected "+mSelected->getName());
      }
    }
  }
}
int main(int, char **)
{
  MyApplication application;
  try
  {
    application.addResourceLocationFrom("../../resources.cfg");
    application.create("../../plugins.cfg","../ogre.cfg","ogre.log",ST_GENERIC);
    application.run();
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

