#include "ogreapplication.h"
#include <OgreErrorDialog.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interface of MyApplication
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class MyApplication : public OgreApplication
{
public:
  MyApplication();
  virtual void init();
};

using namespace Ogre;
using namespace std;

//************************************************************
// Implementation of MyApplication
//************************************************************
MyApplication::MyApplication()
  : OgreApplication("My Application")
{
}
void
MyApplication::init()
{
  mSceneManager->setAmbientLight(ColourValue(0.8f,0.8f,0.8f));
  //////////////////////////////////////////////////////////////
  // Creating our own mesh for an dice
  //////////////////////////////////////////////////////////////
  // Raw data
  static const size_t nbVertices         = 16;
  static const size_t nbFaces            = 12;
  static const size_t nbVerticesPerFaces = 3;
  static const Vector3 positions[nbVertices] =
    {
      Vector3(-0.5f,-0.5f,-0.5f),
      Vector3(+0.5f,-0.5f,-0.5f),
      Vector3(-0.5f,+0.5f,-0.5f),
      Vector3(+0.5f,+0.5f,-0.5f),
      Vector3(-0.5f,-0.5f,+0.5f),
      Vector3(+0.5f,-0.5f,+0.5f),
      Vector3(-0.5f,+0.5f,+0.5f),
      Vector3(+0.5f,+0.5f,+0.5f),
      Vector3(-0.5f,-0.5f,-0.5f),
      Vector3(+0.5f,-0.5f,-0.5f),
      Vector3(-0.5f,+0.5f,-0.5f),
      Vector3(+0.5f,+0.5f,-0.5f),
      Vector3(-0.5f,-0.5f,+0.5f),
      Vector3(+0.5f,-0.5f,+0.5f),
      Vector3(-0.5f,+0.5f,+0.5f),
      Vector3(+0.5f,+0.5f,+0.5f),
    };
  static const Vector2 tcoordinates[nbVertices] =
    {
      Vector2(0.0f,1.0f),
      Vector2(0.0f,0.0f),
      Vector2(0.5f,1.0f),
      Vector2(0.5f,0.0f),      
      Vector2(0.0f,2.0f/3),
      Vector2(0.0f,1.0f/3),
      Vector2(0.5f,2.0f/3),
      Vector2(0.5f,1.0f/3),      
      Vector2(0.5f,1.0f/3),
      Vector2(1.0f,1.0f/3),
      Vector2(0.5f,2.0f/3),
      Vector2(1.0f,2.0f/3),      
      Vector2(0.5f,0.0f),
      Vector2(1.0f,0.0f),
      Vector2(0.5f,1.0f),
      Vector2(1.0f,1.0f),
    };
  static const unsigned short faces[nbFaces][nbVerticesPerFaces] =
    { 
      { 4, 6, 2},{ 2, 0, 4},
      { 1, 3, 7},{ 7, 5, 1},
      { 8, 9,13},{13,12, 8},
      {10,14,15},{15,11,10},
      { 8,10,11},{11, 9, 8},
      { 4, 5, 7},{ 7, 6, 4},
    };
  // Create a mesh
  MeshPtr mesh = MeshManager::getSingleton().createManual("dice.mesh","Custom");
  // We create a submesh per material
  SubMesh* submesh = mesh->createSubMesh();
  // We must create the vertex data, indicating how many vertices there will be
  submesh->useSharedVertices = false;
  submesh->vertexData = new VertexData();
  submesh->vertexData->vertexStart = 0;
  submesh->vertexData->vertexCount = nbVertices;
  // We must now declare what the vertex data contains
  VertexDeclaration* declaration = submesh->vertexData->vertexDeclaration;
  static const unsigned short source = 0;
  size_t offset = 0;
  offset += declaration->addElement(source,offset,VET_FLOAT3,VES_POSITION).getSize();
  offset += declaration->addElement(source,offset,VET_FLOAT3,VES_NORMAL).getSize();
  offset += declaration->addElement(source,offset,VET_FLOAT2,VES_TEXTURE_COORDINATES).getSize();
  // We create the hardware vertex buffer
  HardwareVertexBufferSharedPtr vbuffer =
    HardwareBufferManager::getSingleton().createVertexBuffer(declaration->getVertexSize(source), // == offset
							     submesh->vertexData->vertexCount,   // == nbVertices
							     HardwareBuffer::HBU_STATIC_WRITE_ONLY);
  // No we get access to the buffer to fill it.  During so we record the bounding box.
  AxisAlignedBox aabox;
  float* vdata = static_cast<float*>(vbuffer->lock(HardwareBuffer::HBL_DISCARD));
  for (size_t i=0;i<nbVertices;++i)
  {
    // Position
    Vector3 position = 20.0f*positions[i];
    *vdata++ = position.x;
    *vdata++ = position.y;
    *vdata++ = position.z;
    aabox.merge(position);
    // Normal
    Vector3 normal = position.normalisedCopy();
    *vdata++ = normal.x;
    *vdata++ = normal.y;
    *vdata++ = normal.z;
    // Texture coordinate
    Vector2 tcoordinate = tcoordinates[i];
    *vdata++ = tcoordinate.x;
    *vdata++ = tcoordinate.y;
  }
  vbuffer->unlock();
  submesh->vertexData->vertexBufferBinding->setBinding(source,vbuffer);
  // Creates the index data
  submesh->indexData->indexStart = 0;
  submesh->indexData->indexCount = nbFaces*nbVerticesPerFaces;
  submesh->indexData->indexBuffer =
    HardwareBufferManager::getSingleton().createIndexBuffer(HardwareIndexBuffer::IT_16BIT,
							    submesh->indexData->indexCount,
							    HardwareBuffer::HBU_STATIC_WRITE_ONLY);
  uint16* idata = static_cast<uint16*>(submesh->indexData->indexBuffer->lock(HardwareBuffer::HBL_DISCARD));
  for (size_t i=0;i<nbFaces;++i)
  {
    for (size_t j=0;j<nbVerticesPerFaces;++j)
    {
      *idata++ = faces[i][j];
    }
  }
  submesh->indexData->indexBuffer->unlock();
  // We must indicate the bounding box
  mesh->_setBounds(aabox);  
  mesh->_setBoundingSphereRadius((aabox.getMaximum()-aabox.getMinimum()).length()/2.0);  
  // Finally we set a material to the submesh
  submesh->setMaterialName("Tutorial4/Dice");  
  // And we load the mesh
  mesh->load();

  // Create an entity with the mesh
  Entity *entity = mSceneManager->createEntity("Dice","dice.mesh");
  SceneNode* node = mSceneManager->getRootSceneNode()->createChildSceneNode("DiceNode");
  node->attachObject(entity);

  // Save the mesh for later reuse
  MeshSerializer serializer;
  serializer.exportMesh(mesh.get(),"dice.mesh");
}
int main(int, char **)
{
  MyApplication application;
  try
  {
    application.addResourceLocationFrom("../../resources.cfg");
    application.addResourceLocationFrom("./resources.cfg");
    application.create("../../plugins.cfg","../ogre.cfg","ogre.log",ST_GENERIC);
    application.run();
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

